/*
	render.h

	public interface to refresh functions

	Copyright (C) 1996-1997  Id Software, Inc.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA

	$Id$
*/

#ifndef _RENDER_H
#define _RENDER_H

#include "mathlib.h"
#include "cvar.h"
#include "vid.h"

//=============================================================================

/*

	skin models

*/

typedef enum
{
	LIGHT_NORMAL,
	LIGHT_TINT,
	LIGHT_GLOW,
} r_skin_l_e;

typedef struct r_tri_s
{
	int		vert_indices[3];
} r_tri_t;

typedef struct r_model_s
{
	char		*name;
	int			num_verts;
	int			num_frames;
	vec3_t		**verts;
	int			**vert_norm_indices;
	int			num_triangles;
	r_tri_t		*triangles;
} r_model_t;

/*
 * Note, colors are as follows.
 *
 * num_colors == -1: skin_color is used for all vertices.
 * num_colors == 0: No color is used. (Same as -1 with skin_color 1 1 1)
 * num_colors == num_verts: vert_colors is used.
 *
 * No other values are accepted.
 */
typedef struct r_skin_s
{
	char	*name;

	int		texnum;

	int		num_colors;
	vec4_t	*vert_colors;
	vec4_t	skin_color;
	int		num_texcoords;
	vec2_t	*texcoords;
	int		num_tri_indices;
	int		*tri_indices;
} r_skin_t;

typedef struct r_e_skin_s
{
	r_skin_l_e	type;
	vec4_t		tint;
	r_skin_t	*skin;
} r_e_skin_t;

typedef struct efrag_s
{
	struct mleaf_s		*leaf;
	struct efrag_s		*leafnext;
	struct entity_s		*entity;
	struct efrag_s		*entnext;
} efrag_t;

typedef struct r_entity_s
{
	int				keynum; // for matching entities in different frames
	vec3_t			origin;
	vec3_t			angles; 
	vec3_t			scale;

	r_model_t		*model; // NULL = no model
	int				num_skins;
	r_e_skin_t		**skins;

	// Animation
    float			frame_start_time;
    float			frame_interval;
	int				frame;
	int				visframe;
    int				pose1; 
    int				pose2;

	efrag_t			*efrag; // linked list of efrags (FIXME)

	// Dynamic lighting
	int				dlightframe;
	int				dlightbits;

	// Alias model specific bits.
	int				trivial_accept;

	// Brush model specific bits.
	struct mnode_s			*topnode; // first world node that splits bmodel
} r_entity_t;

// !!! if this is changed, it must be changed in asm_draw.h too !!!
typedef struct
{
	vrect_t		vrect;				// subwindow in video for refresh
									// FIXME: not need vrect next field here?
	vrect_t		aliasvrect;			// scaled Alias version
	int			vrectright, vrectbottom;	// right & bottom screen coords
	int			aliasvrectright, aliasvrectbottom;	// scaled Alias versions
	float		vrectrightedge;			// rightmost right edge we care about,
										//  for use in edge list
	float		fvrectx, fvrecty;		// for floating-point compares
	float		fvrectx_adj, fvrecty_adj; // left and top edges, for clamping
	int			vrect_x_adj_shift20;	// (vrect.x + 0.5 - epsilon) << 20
	int			vrectright_adj_shift20;	// (vrectright + 0.5 - epsilon) << 20
	float		fvrectright_adj, fvrectbottom_adj;
										// right and bottom edges, for clamping
	float		fvrectright;			// rightmost edge, for Alias clamping
	float		fvrectbottom;			// bottommost edge, for Alias clamping
	float		horizontalFieldOfView;	// at Z = 1.0, this many X is visible 
										// 2.0 = 90 degrees
	float		xOrigin;			// should probably always be 0.5
	float		yOrigin;			// between be around 0.3 to 0.5

	vec3_t		vieworg;
	vec3_t		viewangles;

	float		fov_x, fov_y;
	
	int			ambientlight;
} refdef_t;

//
// refresh
//
extern	int		reinit_surfcache;

extern	refdef_t	r_refdef;
extern vec3_t	r_origin, vpn, vright, vup;

extern	struct texture_s	*r_notexture_mip;

extern	r_entity_t	r_worldentity;

void R_Init (void);
void R_Init_Cvars (void);
void R_Textures_Init (void);
void R_InitEfrags (void);
void R_RenderView (void);		// must set r_refdef first
void R_ViewChanged (vrect_t *pvrect, int lineadj, float aspect);
								// called whenever r_refdef or vid change
void R_InitSky (struct texture_s *mt);	// called at level load

void R_AddEfrags (r_entity_t *ent);
void R_RemoveEfrags (r_entity_t *ent);

void R_NewMap (void);

// LordHavoc: relative bmodel lighting
void R_PushDlights (vec3_t entorigin);
void R_DrawWaterSurfaces (void);

//
// surface cache related
//
extern	int		reinit_surfcache;	// if 1, surface cache is currently empty and
extern qboolean	r_cache_thrash;	// set if thrashing the surface cache

void *D_SurfaceCacheAddress (void);
int	D_SurfaceCacheForRes (int width, int height);
void D_FlushCaches (void);
void D_DeleteSurfaceCache (void);
void D_InitCaches (void *buffer, int size);
void R_SetVrect (vrect_t *pvrect, vrect_t *pvrectin, int lineadj);

void R_LoadSkys (char *);

#endif // _RENDER_H
